--[[
	Todo:
		Add class markers
		
	Changelog:
		- 2.1.2 : Fixed issue with blips showing up for max-level characters
		- 2.1.1	: Updated for 5.0.4
		- 2.1	: Added RealID support for /wxp ask (use /wxp ask firstname lastname OR /wxp ask [position in friends list], it will detect the space in the name and use RealID instead of a character name)
		- 2.0.1 : TOC update for 4.3
		- 2.0   : Revamped addon to use blips instead of text
		          Changed config blip to "Example!" so people with the character name "Example" don't break everything
				  Redesigned options panel
				  Removed commands to change settings, just use the config panel
				  
		- 1.4.2 : Fixed problem (probably missing RegisterAddonMessagePrefix) preventing addon from working, and hid the loaded message for real this time
		- 1.4.1 : Updated toc version to 4.2
		- 1.4   : Changed max level from 80 to 85 for Cataclysm and hid the addon load message (who wants those anyway?)
		- 1.3.5 : Added /wxp ask [player] command to get a marker for a person not in your group
		- 1.3.2 : Fixed and bumped toc to 40000 (for real this time)
		- 1.3.1 : Thought I fixed it, didn't
]]--

--- CORE FUNCTIONS ---

function WXP_OnLoad(self) -- Fired when addon is loaded
	SlashCmdList["WXP"] = WXP_Slash;
	SLASH_WXP1 = "/watchxp";
	SLASH_WXP2 = "/wxp";
	self:RegisterEvent("PLAYER_XP_UPDATE")
	self:RegisterEvent("CHAT_MSG_ADDON")
	self:RegisterEvent("ADDON_LOADED")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("CHAT_MSG_BN_WHISPER")
	self:RegisterEvent("CHAT_MSG_BN_WHISPER_INFORM") -- When you send a RealID message
	RegisterAddonMessagePrefix("WXP")
	
	WXP_Version = GetAddOnMetadata("WatchXP","Version")
	WXP_maxLevel = "85"
	WXP_Frames = {}
end

function WXP_OnEvent(self, event, ...) -- Main event handler function
	if (event == "PLAYER_XP_UPDATE") then
		WXP_ExpUpdate()
	end
	
	if (event == "ADDON_LOADED") then
		local addonName = ...;
		if (addonName == "WatchXP") then
			WXP_Options:Hide()
			WXP_Options_Label:Hide()
			WXP_EnumParty()
			SendAddonMessage("WXP","REQ","PARTY")
				
			if (WXP_Settings == nil) then WXP_Settings = {} end
			if (WXP_Settings["Show"] == nil) then WXP_Settings["Show"] = 1 end
			if (WXP_Settings["ShowLabels"] == nil) then WXP_Settings["ShowLabels"] = 1 end
			if (WXP_Settings["ShowLevel"] == nil) then WXP_Settings["ShowLevel"] = 2 end
			if (WXP_Settings["offsetY"] == nil) then WXP_Settings["offsetY"] = 0 end
			if (WXP_Settings["LabelOffsetY"] == nil) then WXP_Settings["LabelOffsetY"] = 0 end
			if (WXP_Settings["Blip"] == nil) then WXP_Settings["Blip"] = 6 end
			if (WXP_Settings["BlipSize"] == nil) then WXP_Settings["BlipSize"] = 16 end
			
			if (WXP_Settings["Colors"] == nil) then
				WXP_Settings["Colors"] = {}
				WXP_Settings["Colors"]["R"] = 1
				WXP_Settings["Colors"]["G"] = 1
				WXP_Settings["Colors"]["B"] = 1
				WXP_Settings["Colors"]["A"] = 1
			end
			
			if (WXP_Settings["Font"] == nil) then
				WXP_Settings["Font"] = {}
				WXP_Settings["Font"]["Face"] = 1
				WXP_Settings["Font"]["Size"] = 12
			end
			
			WXP_InitBlipButtons(WXP_Settings["Blip"])
			
			if (WXP_Settings["Show"] == 0) then
				WXP_Frame:Hide()
			end
			
			-- Add a chat filter so RealID /wxp ask messages aren't shown
			ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_WHISPER",function(self,event,msg,auth,...)
				if (msg:find("WXPASK")) then return true
				elseif (msg:find("WXPNEWXP,")) then return true
				end
			end)
			ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_WHISPER_INFORM",function(self,event,msg,auth,...)
				if (msg:find("WXPASK")) then return true
				elseif (msg:find("WXPNEWXP,")) then return true
				end
			end)
		end
	end
	
	if (event == "CHAT_MSG_ADDON") then
		local addonName, args = ...;
		if (addonName == "WXP") then
			local msgArgs = {strsplit(",", args)}
			
			if (msgArgs[1] == "NEWXP") then
				local name = msgArgs[4]
				local xp = msgArgs[2]
				local xpmax = msgArgs[3]
				local level = msgArgs[5]
				
				if (level ~= WXP_maxLevel and name ~= UnitName("player")) then
					if (WXP_GetFrameFromName(name)) then
						WXP_Update(name,xp,xpmax,level)
					else
						WXP_Create(name,xp,xpmax,level)
					end
				end
			end
			
			if (msgArgs[1] == "REQ") then
				WXP_ExpUpdate()
			end
			
			if (msgArgs[1] == "PING") then
				SendAddonMessage("WXP","PONG,"..UnitName("player")..","..UnitXP("player")..","..UnitXPMax("player")..","..UnitLevel("player"),"WHISPER",msgArgs[2])
			end
			
			if (msgArgs[1] == "PONG") then
				if (msgArgs[5] ~= WXP_maxLevel) then
					WXP_Create(msgArgs[2],msgArgs[3],msgArgs[4],msgArgs[5])
				end
			end
			
			if (msgArgs[1] == "ASK") then
				SendAddonMessage("WXP","NEWXP,"..UnitXP("player")..","..UnitXPMax("player")..","..UnitName("player")..","..UnitLevel("player"),"WHISPER",msgArgs[2])
			end
		end
	end
	
	if (event == "PARTY_MEMBERS_CHANGED") then
		WXP_Wipe(1)
	end
	
	if (event == "CHAT_MSG_BN_WHISPER") then
		local msg,name,_,_,_,_,_,_,_,_,_,_,pid = ...
		if (msg:find("WXPNEWXP,")) then
			local _,xp,xpmax,name,level = strsplit(",", msg)
			
			if (WXP_GetFrameFromName(name)) then
				WXP_Update(name,xp,xpmax,level)
			else
				WXP_Create(name,xp,xpmax,level)
			end
		elseif (msg:find("WXPASK")) then
			BNSendWhisper(pid,"WXPNEWXP,"..UnitXP("player")..","..UnitXPMax("player")..","..UnitName("player")..","..UnitLevel("player"))
		end
	end
end

function WXP_Slash(cmd) -- Command handler function
	if (cmd == "config" or cmd == "options" or cmd == "opt" or cmd == "cfg" or cmd == "win") then
		InterfaceOptionsFrame_OpenToCategory("WatchXP")
	
	elseif (cmd == "toggle") then
		WXP_FrameToggle()
	
	elseif (cmd == "version" or cmd == "v" or cmd == "ver" or cmd == "vers") then
		WXP_Msg("Version: "..WXP_Version)
	
	elseif (cmd == "refresh") then
		WXP_Wipe(1)
		SendAddonMessage("WXP","REQ","PARTY")
	
	elseif (strmatch(cmd,"ask ")) then
		askPlayer = strmatch(cmd,"ask (.*)")
		
		if (strmatch(askPlayer," ")) then -- First and last name
			if (BNConnected()) then
				pid = BNet_GetPresenceID(askPlayer)
				
				if (pid) then
					BNSendWhisper(pid,"WXPASK - Ignore this if you see it :)")
				else
					WXP_Msg("Thet is not a valid RealID friend")
				end
				
			else
				WXP_Msg("Must be connected to RealID to use that feature")
			end
			
		elseif (strmatch(askPlayer,"^%d+$")) then -- Number
			if (BNConnected()) then
				pid = BNGetFriendInfo(askPlayer)
				if (pid) then
					BNSendWhisper(pid,"WXPASK - Ignore this if you see it :)")
				else
					WXP_Msg("Thet is not a valid RealID friend")
				end
				
			else
				WXP_Msg("Must be connected to RealID to use that feature")
			end
			
		else
			SendAddonMessage("WXP","ASK,"..UnitName("player"),"WHISPER",askPlayer)
		end
	
	elseif (cmd == "pos") then
		if (getn(WXP_Frames) > 0) then
			for i=1,getn(WXP_Frames) do
				local xp = WXP_Frames[i]["xp"]
				local xpmax = WXP_Frames[i]["xpmax"]
				local barwidth = math.floor(MainMenuExpBar:GetWidth())
				local barpos = math.floor(MainMenuExpBar:GetLeft())
				local pos = math.floor(barwidth * (xp / xpmax))
				
				print("-- "..WXP_Frames[i]["name"].." --")
				print("  xp - "..xp.." / "..xpmax)
				print("  barwidth - "..barwidth)
				print("  barpos - "..barpos)
				print("  pos - "..pos)
			end
		else
			print("No blips!")
		end
		
	else
		WXP_Msg("|cffffc445/wxp config|r : Show the configuration panel")
		WXP_Msg("|cffffc445/wxp ask player|r : Request a marker for a player not in your group (they still need WatchXP)")
		WXP_Msg("|cffffc445/wxp ask FirstName LastName|r : Request a marker for a RealID friend (they still need WatchXP)")
		WXP_Msg("|cffffc445/wxp ask n|r : Request a marker for the |cffffc445n|rth online RealID friend in your friends list (they still need WatchXP)")
		WXP_Msg("|cffffc445/wxp toggle|r : Toggle displaying of WatchXP")
		WXP_Msg("|cffffc445/wxp refresh|r : Refreshes the display and forces an update from all party members. Also removes leftofver markers or markers added with /wxp ask")
		WXP_Msg("|cffffc445/wxp version|r : Display addon version")
	end
end

function WXP_Update(name,xp,xpmax,level) -- Update the position of an existing blip
	local num = WXP_GetFrameFromName(name)
	
	WXP_RefreshMarker(num,name,xp,xpmax,level)
	
	WXP_Frames[num]["fontstring"]:ClearAllPoints()
	WXP_Frames[num]["fontstring"]:SetPoint("LEFT",WXP_Frames[num]["frame"],"RIGHT",0,WXP_Settings["LabelOffsetY"])
	
	WXP_Frames[num]["xp"] = xp
	WXP_Frames[num]["xpmax"] = xpmax
	WXP_Frames[num]["level"] = level
	WXP_Draw(num)
end

function WXP_RefreshMarker(num,name,xp,xpmax,level) -- Refresh the blip text, to update if the level is shown or not
	if (WXP_Settings["ShowLevel"] == 0) then -- Never show level
		WXP_Frames[num]["fontstring"]:SetText(name)
	elseif (WXP_Settings["ShowLevel"] == 1) then  -- Always show level
		WXP_Frames[num]["fontstring"]:SetText(name.." ("..level..")")
	else  -- Show level if different
		if (level ~= UnitLevel("Player")) then
			WXP_Frames[num]["fontstring"]:SetText(name.." ("..level..")")
		else
			WXP_Frames[num]["fontstring"]:SetText(name)
		end
	end
end

function WXP_Draw(num) -- Redraw/refresh an existing blip
	local name = WXP_Frames[num]["name"]
	local level = WXP_Frames[num]["level"]
	local xp = WXP_Frames[num]["xp"]
	local xpmax = WXP_Frames[num]["xpmax"]
	local barwidth = 0
	local xpwidth = 0
	
	WXP_RefreshMarker(num,name,xp,xpmax,level)
	
	if (level == nil) then
		level = "?"
	end
	
	if (name == nil) then
		name = "Unknown"
	end
	
	if (level ~= WXP_maxLevel) then
		barwidth = MainMenuExpBar:GetWidth()
		
		if xpmax == 0 then
			xpwidth = 0
		else
			xpwidth = barwidth * (xp / xpmax)
		end
		
		_G["WXP_Frame_"..num]:SetPoint("CENTER",WXP_Frame,"LEFT",xpwidth,WXP_Settings["offsetY"])
		
		if (WXP_CheckFlip(num)) then
			WXP_Frames[num]["fontstring"]:ClearAllPoints()
			WXP_Frames[num]["fontstring"]:SetPoint("RIGHT",WXP_Frames[num]["frame"],"LEFT",0,WXP_Settings["LabelOffsetY"])
		end
	end
end

function WXP_Create(name,xp,xpmax,level) -- Create a new player blip
	local num,new = WXP_FindFirstEmpty()
	
	if (new) then
		WXP_Frames[num] = {}
		
		new = CreateFrame("Frame","WXP_Frame_"..num,WXP_Frame,nil,num)
		new:SetPoint("CENTER",WXP_Frame,"LEFT",0,WXP_Settings["offsetY"])
		new:SetSize(WXP_Settings["BlipSize"],WXP_Settings["BlipSize"])
		new:SetScript("OnEnter",WXP_OnEnter)
		new:SetScript("OnLeave",WXP_OnLeave)
		
		newtex = new:CreateTexture("WXP_Tex_"..num,"ARTWORK")
		new:SetSize(WXP_Settings["BlipSize"],WXP_Settings["BlipSize"])
		newtex:SetAllPoints()
		newtex:SetTexture("Interface\\Addons\\WatchXP\\Icons.tga")
		newtex:SetTexCoord(WXP_GetIconCoord(WXP_GetIconXYFromID(WXP_Settings["Blip"])))		
		
		newfont = new:CreateFontString("WXP_Text_"..num,"ARTWORK","GameFontNormal")
		
		if (WXP_Settings["ShowLevel"] == 0) then -- Never show level
			newfont:SetText(name)
		elseif (WXP_Settings["ShowLevel"] == 1) then  -- Always show level
			newfont:SetText(name.." ("..level..")")
		else  -- Show level if different
			if (level ~= UnitLevel("Player")) then
				newfont:SetText(name.." ("..level..")")
			else
				newfont:SetText(name)
			end
		end
		
		newfont:SetPoint("LEFT",new,"RIGHT",0,WXP_Settings["LabelOffsetY"])
		newfont:SetTextColor(WXP_Settings["Colors"]["R"],WXP_Settings["Colors"]["G"],WXP_Settings["Colors"]["B"],WXP_Settings["Colors"]["A"])
		
		if (WXP_Settings["Font"]["Face"] == 1) then
			newfont:SetFont("Fonts\\FRIZQT__.TTF",WXP_Settings["Font"]["Size"])
		elseif (WXP_Settings["Font"]["Face"] == 2) then
			newfont:SetFont("Fonts\\\ARIALN.TTF",WXP_Settings["Font"]["Size"])
		elseif (WXP_Settings["Font"]["Face"] == 3) then
			newfont:SetFont("Fonts\\MORPHEUS.TTF",WXP_Settings["Font"]["Size"])
		elseif (WXP_Settings["Font"]["Face"] == 4) then
			newfont:SetFont("Fonts\\SKURRI.TTF",WXP_Settings["Font"]["Size"])
		else
			newfont:SetFont("Fonts\\FRIZQT__.TTF",WXP_Settings["Font"]["Size"])
		end
		
		if (WXP_Settings["ShowLabels"] == 0) then
			newfont:Hide()
		end
		
		WXP_Frames[num]["frame"] = new
		WXP_Frames[num]["fontstring"] = newfont
		WXP_Frames[num]["tex"] = newtex
	else
		WXP_Frames[num]["frame"]:Show()
	end
	
	WXP_Frames[num]["name"] = name
	WXP_Frames[num]["xp"] = xp
	WXP_Frames[num]["xpmax"] = xpmax
	WXP_Frames[num]["level"] = level
	
	WXP_Draw(num)
end

function WXP_Destroy(num) -- Destroy the given frame
	if (num) then
		WXP_Frames[num]["name"] = ""
		WXP_Frames[num]["xp"] = ""
		WXP_Frames[num]["xpmax"] = ""
		WXP_Frames[num]["level"] = ""
		WXP_Frames[num]["frame"]:Hide()
	end
end

function WXP_FrameToggle() -- Toggle showing/hiding WXP
	if (WXP_Settings["Show"] == 1) then
		WXP_Msg("Hiding WatchXP")
		WXP_Frame:Hide()
		WXP_Settings["Show"] = 0
	else
		WXP_Msg("Showing WatchXP")
		WXP_Frame:Show()
		WXP_Settings["Show"] = 1
	end
end

function WXP_EnumParty() -- Send out a PING to all party members
	for i=1,GetNumSubgroupMembers() do
		SendAddonMessage("WXP","PING,"..UnitName("player"),"WHISPER",UnitName("party"..i))
	end
end

function WXP_Wipe(redo) -- Remove all blips, optionally request new ones
	for i=1,getn(WXP_Frames) do
		WXP_Frames[i]["name"] = ""
		WXP_Frames[i]["xp"] = ""
		WXP_Frames[i]["xpmax"] = ""
		WXP_Frames[i]["level"] = ""
	end
	
	local children = {WXP_Frame:GetChildren()}
	for _, child in pairs(children) do
		if(child:IsObjectType("Frame")) then
			child:Hide()
		end
	end
	if (redo) then
		SendAddonMessage("WXP","REQ","PARTY")
	end
end

function WXP_ExpUpdate() -- Send out new exp value to party
	SendAddonMessage("WXP","NEWXP,"..UnitXP("player")..","..UnitXPMax("player")..","..UnitName("player")..","..UnitLevel("player"),"PARTY")
end

-- HELPER FUNCTIONS ---

function WXP_Msg(msg) -- Message output function
	DEFAULT_CHAT_FRAME:AddMessage("|cffd2b48c[WatchXP]|r "..msg)
end

function WXP_FindFirstEmpty() -- Find the first available frame (or tell it to create a new one of none are empty)
	if (getn(WXP_Frames) == 0) then
		return 1,true
	else
		for i=1,getn(WXP_Frames) do
			if (WXP_Frames[i]["name"] == "") then
				return i,false
			end
		end
		return getn(WXP_Frames)+1,true
	end
end

function WXP_GetFrameFromName(name) -- Find a frame with a given player name
	if (getn(WXP_Frames) == 0) then
		return false
	else
		for i=1,getn(WXP_Frames) do
			if (WXP_Frames[i]["name"] == name) then
				return i
			end
		end
		return false
	end
end

function WXP_GetIconCoord(x,y) -- Get coordinates of an icon at x,y position

	local l = x * .125
	local t = y * .125
	local r = l + .125
	local b = t + .125

	return l,r,t,b
end

function WXP_GetIconXYFromID(id) -- Return the x,y position of a blip of given id
	local x = 0
	local y = 0
	
	if     (id == 0)  then x = 0; y = 0
	elseif (id == 1)  then x = 1; y = 0
	elseif (id == 2)  then x = 2; y = 0
	elseif (id == 3)  then x = 3; y = 0
	elseif (id == 4)  then x = 4; y = 0
	elseif (id == 5)  then x = 5; y = 0
	elseif (id == 6)  then x = 6; y = 0
	elseif (id == 7)  then x = 7; y = 0
	elseif (id == 8)  then x = 0; y = 1
	elseif (id == 9)  then x = 1; y = 1
	elseif (id == 10) then x = 2; y = 1
	elseif (id == 11) then x = 3; y = 1
	elseif (id == 12) then x = 4; y = 1
	elseif (id == 13) then x = 5; y = 1
	elseif (id == 14) then x = 6; y = 1
	elseif (id == 15) then x = 7; y = 1
	elseif (id == 16) then x = 0; y = 2
	elseif (id == 17) then x = 1; y = 2
	elseif (id == 18) then x = 2; y = 2
	elseif (id == 19) then x = 3; y = 2
	elseif (id == 20) then x = 4; y = 2
	elseif (id == 21) then x = 5; y = 2
	elseif (id == 22) then x = 6; y = 2
	elseif (id == 23) then x = 7; y = 2
	elseif (id == 24) then x = 0; y = 3
	elseif (id == 25) then x = 1; y = 3
	elseif (id == 26) then x = 2; y = 3
	elseif (id == 27) then x = 3; y = 3
	elseif (id == 28) then x = 4; y = 3
	elseif (id == 29) then x = 5; y = 3
	
	elseif (id == 30) then x = 0; y = 4
	elseif (id == 31) then x = 1; y = 4
	elseif (id == 32) then x = 2; y = 4
	elseif (id == 33) then x = 3; y = 4
	elseif (id == 34) then x = 4; y = 4
	elseif (id == 35) then x = 5; y = 4
	elseif (id == 36) then x = 6; y = 4
	elseif (id == 37) then x = 7; y = 4
	end
	
	return x,y
end

function WXP_CheckFlip(num) -- Check if a label should be flipped (near end of bar or next to another blip)
		local barright = math.floor(WXP_Frame:GetRight())
		local stringwidth = math.floor(WXP_Frames[num]["fontstring"]:GetStringWidth())
		local blipright = math.floor(WXP_Frames[num]["frame"]:GetRight())
		local pxremain = barright-(blipright+stringwidth)
		
		local close = false
		
		if (getn(WXP_Frames) > 0) then
			for i=1,getn(WXP_Frames) do
				if (i ~= num) then
					local pos = math.floor(WXP_Frames[i]["frame"]:GetLeft())
					if (WXP_Frames[num]["frame"]:GetLeft() + stringwidth > pos) and (WXP_Frames[num]["frame"]:GetLeft() < WXP_Frames[i]["frame"]:GetLeft()) then
						close = true
						break
					end
				end
			end
		end
		
		if (pxremain < 0) or (close) then
			return true
		else
			return false
		end
end

--- UI STUFF ---

function WXP_HideLabels() -- Hide all labels
	if (getn(WXP_Frames) > 0) then
		for i=1,getn(WXP_Frames) do
			WXP_Frames[i]["fontstring"]:Hide()
		end
	end
end

function WXP_ShowLabels() -- Show all labels
	if (getn(WXP_Frames) > 0) then
		for i=1,getn(WXP_Frames) do
			WXP_Frames[i]["fontstring"]:Show()
		end
	end
end

function WXP_OnEnter() -- Fired when mouse enters a blip
	num = GetMouseFocus():GetID()
	local name = WXP_Frames[num]["name"] or "Unknown"
	local xp = WXP_Frames[num]["xp"] or 0
	local xpmax = WXP_Frames[num]["xpmax"] or 0
	local level = WXP_Frames[num]["level"] or 0
	
	local pct = math.floor((xp/xpmax)*100)
	
	GameTooltip:SetOwner(WXP_Frame,"ANCHOR_CURSOR")
	GameTooltip:SetText(name)
	GameTooltip:AddLine("Level: "..level)
	GameTooltip:AddLine("XP: "..xp.." / "..xpmax.." ("..pct.."%)")
	GameTooltip:Show()
end

function WXP_OnLeave() -- Fired when mouse leaves a blip
	GameTooltip:Hide()
end

function WXP_OptionsLoad(self) -- Initialize Options panel
	if (self:GetName() == "WXP_Options") then
		self.name = "WatchXP"
		self.default = WXP_Defaults
		InterfaceOptions_AddCategory(self)
	else
		self.name = "Labels"
		self.parent = "WatchXP"
		self.default = WXP_Defaults
		InterfaceOptions_AddCategory(self)
	end
end

function WXP_OptionsShow() -- Fired when options panel opens
	WXP_Create("Example!",1,4,1)
end

function WXP_OptionsHide() -- Fired when options panel closes
	WXP_Destroy(WXP_GetFrameFromName("Example!"))
end

function WXP_Opt_GetColor() -- Fired when color picker color is updated
	WXP_Settings["Colors"]["R"], WXP_Settings["Colors"]["G"], WXP_Settings["Colors"]["B"] = ColorPickerFrame:GetColorRGB()
	if (getn(WXP_Frames) > 0) then
		for i=1,getn(WXP_Frames) do
			WXP_Frames[i]["fontstring"]:SetTextColor(WXP_Settings["Colors"]["R"],WXP_Settings["Colors"]["G"],WXP_Settings["Colors"]["B"],WXP_Settings["Colors"]["A"])
		end
	end
end

function WXP_ShowColorPicker() -- Show the color picker
	local r = WXP_Settings["Colors"]["R"]
	local g = WXP_Settings["Colors"]["G"]
	local b = WXP_Settings["Colors"]["B"]
	local a = WXP_Settings["Colors"]["A"]
	ColorPickerFrame.hasOpacity = true;
	ColorPickerFrame.opacity = 1 - a;
	ColorPickerFrame.func = WXP_Opt_GetColor
	ColorPickerFrame.opacityFunc = WXP_Opt_GetOpacity
	ColorPickerFrame:SetColorRGB(r, g, b);
	ColorPickerFrame:Show();
end

function WXP_Opt_GetOpacity() -- Fired when color picker opacity is updated
	WXP_Settings["Colors"]["A"] = 1 - OpacitySliderFrame:GetValue()
	if (getn(WXP_Frames) > 0) then
		for i=1,getn(WXP_Frames) do
			WXP_Frames[i]["fontstring"]:SetTextColor(WXP_Settings["Colors"]["R"],WXP_Settings["Colors"]["G"],WXP_Settings["Colors"]["B"],WXP_Settings["Colors"]["A"])
		end
	end
end

function WXP_ButtonClick(self) -- Fired when an options panel widget is used
	if (self:GetObjectType() ~= "Slider") then
		PlaySound("igMainMenuOptionCheckBoxOn")
	end
	
	if (self:GetName() == "WXP_OptBut_Show") then
		if (WXP_OptBut_Show:GetChecked()) then
			WXP_Frame:Show()
			WXP_Settings["Show"] = 1
		else
			WXP_Frame:Hide()
			WXP_Settings["Show"] = 0
		end
	
	elseif (self:GetName() == "WXP_OptBut_ShowLabels") then
		if (WXP_OptBut_ShowLabels:GetChecked()) then
			WXP_ShowLabels()
			WXP_Settings["ShowLabels"] = 1
		else
			WXP_HideLabels()
			WXP_Settings["ShowLabels"] = 0
		end
	
	elseif (self:GetName() == "WXP_OptBut_ShowLevel1") then -- Never show level
		WXP_Settings["ShowLevel"] = 0
		WXP_OptBut_ShowLevel1:SetChecked(1)
		WXP_OptBut_ShowLevel2:SetChecked(nil)
		WXP_OptBut_ShowLevel3:SetChecked(nil)
		if (getn(WXP_Frames) > 0) then
			for i=1,getn(WXP_Frames) do
				WXP_Frames[i]["fontstring"]:SetText(WXP_Frames[i]["name"])
			end
		end
	
	elseif (self:GetName() == "WXP_OptBut_ShowLevel2") then -- Show level if different
		WXP_Settings["ShowLevel"] = 2
		WXP_OptBut_ShowLevel1:SetChecked(nil)
		WXP_OptBut_ShowLevel2:SetChecked(1)
		WXP_OptBut_ShowLevel3:SetChecked(nil)
		if (getn(WXP_Frames) > 0) then
			for i=1,getn(WXP_Frames) do
				if (WXP_Frames[i]["level"] ~= UnitLevel("Player")) then
					WXP_Frames[i]["fontstring"]:SetText(WXP_Frames[i]["name"].." ("..WXP_Frames[i]["level"]..")")
				else
					WXP_Frames[i]["fontstring"]:SetText(WXP_Frames[i]["name"])
				end
			end
		end
	
	elseif (self:GetName() == "WXP_OptBut_ShowLevel3") then -- Always
		WXP_Settings["ShowLevel"] = 1
		WXP_OptBut_ShowLevel1:SetChecked(nil)
		WXP_OptBut_ShowLevel2:SetChecked(nil)
		WXP_OptBut_ShowLevel3:SetChecked(1)
		if (getn(WXP_Frames) > 0) then
			for i=1,getn(WXP_Frames) do
				WXP_Frames[i]["fontstring"]:SetText(WXP_Frames[i]["name"].." ("..WXP_Frames[i]["level"]..")")
			end
		end
	
	elseif (self:GetName() == "WXP_OptBut_Font1") then -- Never show level
		WXP_Settings["Font"]["Face"] = 1
		WXP_OptBut_Font1:SetChecked(1)
		WXP_OptBut_Font2:SetChecked(nil)
		WXP_OptBut_Font3:SetChecked(nil)
		WXP_OptBut_Font4:SetChecked(nil)
		if (getn(WXP_Frames) > 0) then
			for i=1,getn(WXP_Frames) do
				WXP_Frames[i]["fontstring"]:SetFont("Fonts\\FRIZQT__.TTF",WXP_Settings["Font"]["Size"])
			end
		end
	
	elseif (self:GetName() == "WXP_OptBut_Font2") then -- Never show level
		WXP_Settings["Font"]["Face"] = 2
		WXP_OptBut_Font1:SetChecked(nil)
		WXP_OptBut_Font2:SetChecked(1)
		WXP_OptBut_Font3:SetChecked(nil)
		WXP_OptBut_Font4:SetChecked(nil)
		if (getn(WXP_Frames) > 0) then
			for i=1,getn(WXP_Frames) do
				WXP_Frames[i]["fontstring"]:SetFont("Fonts\\ARIALN.TTF",WXP_Settings["Font"]["Size"])
			end
		end
	
	elseif (self:GetName() == "WXP_OptBut_Font3") then -- Never show level
		WXP_Settings["Font"]["Face"] = 3
		WXP_OptBut_Font1:SetChecked(nil)
		WXP_OptBut_Font2:SetChecked(nil)
		WXP_OptBut_Font3:SetChecked(1)
		WXP_OptBut_Font4:SetChecked(nil)
		if (getn(WXP_Frames) > 0) then
			for i=1,getn(WXP_Frames) do
				WXP_Frames[i]["fontstring"]:SetFont("Fonts\\MORPHEUS.TTF",WXP_Settings["Font"]["Size"])
			end
		end
	
	elseif (self:GetName() == "WXP_OptBut_Font4") then -- Never show level
		WXP_Settings["Font"]["Face"] = 4
		WXP_OptBut_Font1:SetChecked(nil)
		WXP_OptBut_Font2:SetChecked(nil)
		WXP_OptBut_Font3:SetChecked(nil)
		WXP_OptBut_Font4:SetChecked(1)
		if (getn(WXP_Frames) > 0) then
			for i=1,getn(WXP_Frames) do
				WXP_Frames[i]["fontstring"]:SetFont("Fonts\\SKURRI.TTF",WXP_Settings["Font"]["Size"])
			end
		end
		
	elseif (self:GetName() == "WXP_OptBut_OffsetY") then
		if (WXP_Settings) then
			WXP_Settings["offsetY"] = self:GetValue()
			if (getn(WXP_Frames) > 0) then
				for i=1,getn(WXP_Frames) do
					local _,_,_,xoff = WXP_Frames[i]["frame"]:GetPoint()
					WXP_Frames[i]["frame"]:SetPoint("CENTER",WXP_Frame,"LEFT",xoff,WXP_Settings["offsetY"])
				end
			end
		end
	
	elseif (self:GetName() == "WXP_OptBut_BlipSize") then
		if (WXP_Settings) then
			WXP_Settings["BlipSize"] = self:GetValue()
			if (getn(WXP_Frames) > 0) then
				for i=1,getn(WXP_Frames) do
					WXP_Frames[i]["frame"]:SetSize(WXP_Settings["BlipSize"],WXP_Settings["BlipSize"])
					WXP_Frames[i]["tex"]:SetSize(WXP_Settings["BlipSize"],WXP_Settings["BlipSize"])
				end
			end
		end
	
	elseif (self:GetName() == "WXP_OptBut_LabelOffsetY") then
		if (WXP_Settings) then
			WXP_Settings["LabelOffsetY"] = self:GetValue()
			if (getn(WXP_Frames) > 0) then
				for i=1,getn(WXP_Frames) do
					newfont:SetPoint("LEFT",new,"RIGHT",0,0)
					WXP_Frames[i]["fontstring"]:SetPoint("LEFT",WXP_Frames[i]["frame"],"RIGHT",0,WXP_Settings["LabelOffsetY"])
				end
			end
		end
		
	elseif (self:GetName() == "WXP_OptBut_FontSize") then
		if (WXP_Settings) then
			WXP_Settings["Font"]["Size"] = self:GetValue()
			if (getn(WXP_Frames) > 0) then
				for i=1,getn(WXP_Frames) do
					local fname, _, _ = WXP_Frames[i]["fontstring"]:GetFont()
					WXP_Frames[i]["fontstring"]:SetFont(fname,self:GetValue())
				end
			end
		end
	end
end

function WXP_SliderInit(slider) -- Initialize sliders
	local s1 = _G[slider]
	if (s1:GetName() == "WXP_OptBut_FontSize") then
		s1.tooltipText = "Change the size of the label text"
		_G[slider.."Low"]:SetText("4")
		_G[slider.."High"]:SetText("36")
		_G[slider.."Text"]:SetText("12")
	
	elseif (s1:GetName() == "WXP_OptBut_OffsetY") then
		s1.tooltipText = "Change the vertical offset of the blips"
		_G[slider.."Low"]:SetText("-100")
		_G[slider.."High"]:SetText("100")
		_G[slider.."Text"]:SetText("0")
	
	elseif (s1:GetName() == "WXP_OptBut_LabelOffsetY") then
		s1.tooltipText = "Change the vertical offset of the labels"
		_G[slider.."Low"]:SetText("-100")
		_G[slider.."High"]:SetText("100")
		_G[slider.."Text"]:SetText("0")
	
	elseif (s1:GetName() == "WXP_OptBut_BlipSize") then
		s1.tooltipText = "Size of the blips"
		_G[slider.."Low"]:SetText("8")
		_G[slider.."High"]:SetText("64")
		_G[slider.."Text"]:SetText("0")
	end
end

function WXP_BlipsClick(self,id) -- Fired when a blip button is clicked
	local x,y = WXP_GetIconXYFromID(id)
	WXP_Settings["Blip"] = id
	WXP_InitBlipButtons(id)
	
	if (getn(WXP_Frames) > 0) then
		for i=1,getn(WXP_Frames) do
			WXP_Frames[i]["tex"]:SetTexCoord(WXP_GetIconCoord(x,y))
		end
	end
end

function WXP_InitBlipButtons(id) -- Initialize blip buttons (hide all, show given id)
	WXP_OptBut_Icon0:SetAlpha("0")
	WXP_OptBut_Icon1:SetAlpha("0")
	WXP_OptBut_Icon2:SetAlpha("0")
	WXP_OptBut_Icon3:SetAlpha("0")
	WXP_OptBut_Icon4:SetAlpha("0")
	WXP_OptBut_Icon5:SetAlpha("0")
	WXP_OptBut_Icon6:SetAlpha("0")
	WXP_OptBut_Icon7:SetAlpha("0")
	WXP_OptBut_Icon8:SetAlpha("0")
	WXP_OptBut_Icon9:SetAlpha("0")
	WXP_OptBut_Icon10:SetAlpha("0")
	WXP_OptBut_Icon11:SetAlpha("0")
	WXP_OptBut_Icon12:SetAlpha("0")
	WXP_OptBut_Icon13:SetAlpha("0")
	WXP_OptBut_Icon14:SetAlpha("0")
	WXP_OptBut_Icon15:SetAlpha("0")
	WXP_OptBut_Icon16:SetAlpha("0")
	WXP_OptBut_Icon17:SetAlpha("0")
	WXP_OptBut_Icon18:SetAlpha("0")
	WXP_OptBut_Icon19:SetAlpha("0")
	WXP_OptBut_Icon20:SetAlpha("0")
	WXP_OptBut_Icon21:SetAlpha("0")
	WXP_OptBut_Icon22:SetAlpha("0")
	WXP_OptBut_Icon23:SetAlpha("0")
	WXP_OptBut_Icon24:SetAlpha("0")
	WXP_OptBut_Icon25:SetAlpha("0")
	WXP_OptBut_Icon26:SetAlpha("0")
	WXP_OptBut_Icon27:SetAlpha("0")
	WXP_OptBut_Icon28:SetAlpha("0")
	WXP_OptBut_Icon29:SetAlpha("0")
	WXP_OptBut_Icon30:SetAlpha("0")
	WXP_OptBut_Icon31:SetAlpha("0")
	WXP_OptBut_Icon32:SetAlpha("0")
	WXP_OptBut_Icon33:SetAlpha("0")
	WXP_OptBut_Icon34:SetAlpha("0")
	WXP_OptBut_Icon35:SetAlpha("0")
	WXP_OptBut_Icon36:SetAlpha("0")
	WXP_OptBut_Icon37:SetAlpha("0")
	_G["WXP_OptBut_Icon"..id]:SetAlpha("0.65")
end

function WXP_Defaults() -- Reset all settings to default when "Defaults" button is clicked
	-- Show
	WXP_Settings["Show"] = 1
	WXP_OptBut_Show:SetChecked(1)
	
	-- Show Label
	WXP_Settings["ShowLabels"] = 1
	WXP_OptBut_ShowLabels:SetChecked(1)
	
	-- Blip Offset
	WXP_Settings["offsetY"] = 0
	WXP_OptBut_OffsetY:SetValue(0)
	
	-- Label Offset
	WXP_Settings["LabelOffsetY"] = 0
	WXP_OptBut_LabelOffsetY:SetValue(0)
	
	-- Blip Size
	WXP_Settings["BlipSize"] = 16
	WXP_OptBut_BlipSize:SetValue(16)
	
	-- Blip
	WXP_Settings["Blip"] = 6
	WXP_InitBlipButtons(6)
	
	WXP_Settings["Colors"]["R"] = 1
	WXP_Settings["Colors"]["G"] = 1
	WXP_Settings["Colors"]["B"] = 1
	WXP_Settings["Colors"]["A"] = 1
	
	-- Font
	WXP_Settings["Font"]["Face"] = 1
	WXP_Settings["Font"]["Size"] = 12
	WXP_OptBut_Font1:SetChecked(1)
	WXP_OptBut_Font2:SetChecked(nil)
	WXP_OptBut_Font3:SetChecked(nil)
	WXP_OptBut_Font4:SetChecked(nil)
	WXP_OptBut_FontSize:SetValue(12)
	
	-- Show Level
	WXP_Settings["ShowLevel"] = 2
	WXP_OptBut_ShowLevel1:SetChecked(nil)
	WXP_OptBut_ShowLevel2:SetChecked(1)
	WXP_OptBut_ShowLevel3:SetChecked(nil)
	
	-- change blip, change text color
	
	if (getn(WXP_Frames) > 0) then
			for i=1,getn(WXP_Frames) do
				if (WXP_Frames[i]["level"] ~= UnitLevel("Player")) then
					WXP_Frames[i]["fontstring"]:SetText(WXP_Frames[i]["name"].." ("..WXP_Frames[i]["level"]..")")
				else
					WXP_Frames[i]["fontstring"]:SetText(WXP_Frames[i]["name"])
				end

				WXP_Frame:Show()
				local _,_,_,xoff = WXP_Frames[i]["frame"]:GetPoint()
				WXP_Frames[i]["frame"]:SetPoint("CENTER",WXP_Frame,"LEFT",xoff,0)
				WXP_Frames[i]["frame"]:SetSize(WXP_Settings["BlipSize"],16)
				WXP_Frames[i]["tex"]:SetSize(WXP_Settings["BlipSize"],16)
				WXP_Frames[i]["fontstring"]:SetPoint("LEFT",WXP_Frames[i]["frame"],"RIGHT",0,0)
				WXP_Frames[i]["fontstring"]:SetFont("Fonts\\FRIZQT__.TTF",12)
				WXP_Frames[i]["tex"]:SetTexCoord(WXP_GetIconCoord(WXP_GetIconXYFromID(6)))
				WXP_Frames[i]["fontstring"]:SetTextColor(255,255,255,1)
			end
		end
end

--- MISC HELPER FUNCTIONS ---

function print_r (t,indent,done) -- Prints out a table
  done = done or {}
  indent = indent or ''
  local nextIndent -- Storage for next indentation value
  for key, value in pairs (t) do
    if type (value) == "table" and not done [value] then
      nextIndent = nextIndent or
          (indent .. string.rep(' ',string.len(tostring (key))+2))
          -- Shortcut conditional allocation
      done [value] = true
      print (indent .. "[" .. tostring (key) .. "] => Table {");
      print  (nextIndent .. "{");
      print_r (value, nextIndent .. string.rep(' ',2), done)
      print  (nextIndent .. "}");
    else
      print  (indent .. "[" .. tostring (key) .. "] => " .. tostring (value).."")
    end
  end
end